@@ -9,32 +9,28 @@ use bytes;
 use Math::BigInt;
 use Socket;
 
-our $ERROR     = '';
+our $ERROR     = q();
 our %SNMP_TYPE = (
-                     'INTEGER'   => [ 0x02, \&uint   ],
-                     'STRING'    => [ 0x04, \&string ],
-                     'NULLOBJ'   => [ 0x05, sub {}   ],
-                     'IPADDRESS' => [ 0x40, \&ip     ],
-                     'COUNTER'   => [ 0x41, \&uint   ],
-                     'UNSIGNED'  => [ 0x42, \&uint   ],
-                     'TIMETICKS' => [ 0x43, \&uint   ],
-                     'OPAQUE'    => [ 0x44, \&uint   ],
-                     'COUNTER64' => [ 0x46, \&bigint ],
-                 );
+    INTEGER   => [ 0x02, \&uint   ],
+    STRING    => [ 0x04, \&string ],
+    NULLOBJ   => [ 0x05, sub {}   ],
+    IPADDRESS => [ 0x40, \&ip     ],
+    COUNTER   => [ 0x41, \&uint   ],
+    UNSIGNED  => [ 0x42, \&uint   ],
+    TIMETICKS => [ 0x43, \&uint   ],
+    OPAQUE    => [ 0x44, \&uint   ],
+    COUNTER64 => [ 0x46, \&bigint ],
+);
 
 
-sub byte_size { #=============================================================
-    return $DOCSIS::Perl::BYTE_SIZE{lc shift} || 0;
-}
-
 sub snmp_type { #=============================================================
     return $SNMP_TYPE{uc shift} || $SNMP_TYPE{'STRING'};
 }
 
 sub snmp_oid { #==============================================================
 
-    ### init
-    my @input_oid = split /\./, shift;
+    my $string    = shift or return;
+    my @input_oid = split /\./, $string;
     my $subid     = 0;
     my @encoded_oid;
 
@@ -61,32 +57,30 @@ sub snmp_oid { #==============================================================
         }
     }
 
-    ### the end
-    return @encoded_oid;
+    return wantarray ? @encoded_oid : \@encoded_oid;
 }
 
 sub snmp_object { #===========================================================
 
-    ### init
-    my $obj   = shift()->{'value'};
-    my @oid   = snmp_oid($obj->{'oid'});
-    my $type  = snmp_type($obj->{'type'});
-    my @value = $type->[1]->({ value => $obj->{'value'}, snmp => 1 });
+    my $obj    = shift->{'value'} or return;
+    my @oid    = snmp_oid($obj->{'oid'});
+    my $type   = snmp_type($obj->{'type'});
+    my @value  = $type->[1]->({ value => $obj->{'value'}, snmp => 1 });
+    my $length = int(@oid) + int(@value) + 4;
+    
+    my @ret = (
+      #-type--------length-------value-----type---
+        48,         $length,             # object
+        6,          int(@oid),   @oid,   # oid
+        $type->[0], int(@value), @value, # value
+    );
 
-    ### the end
-    return 48,
-           int(@oid) + int(@value) + 4,
-           6,
-           int(@oid),   @oid,
-           $type->[0],
-           int(@value), @value,
-           ;
+    return wantarray ? @ret : \@ret;
 }
 
 sub bigint { #================================================================
 
-    ### init
-    my $int64    = Math::BigInt->new(shift()->{'value'});
+    my $int64    = Math::BigInt->new(shift->{'value'});
     my $negative = $int64 < 0;
     my @bytes    = $negative ? (0x80) : ();
 
@@ -97,19 +91,19 @@ sub bigint { #================================================================
         unshift @bytes, $value;
     }
 
-    ### the end
-    return @bytes || (0);
+    ### bytes need a value
+    @bytes = (0) unless(@bytes);
+
+    return wantarray ? @bytes : \@bytes;
 }
 
 sub uint { #==================================================================
 
-    ### init
     my $obj      = shift;
     my $int      = $obj->{'value'} || 0;
     my $negative = $int < 0;
     my @bytes;
 
-
     while($int) {
         my $value  = $int & 0xff;
         $int     >>= 8;
@@ -131,13 +125,11 @@ sub uint { #==================================================================
         unshift @bytes, 0 if(!$negative and $bytes[0] > 0x79);
     }
 
-    ### the end
-    return @bytes;
+    return wantarray ? @bytes : \@bytes;
 }
 
 sub ushort { #================================================================
 
-    ### init
     my $obj      = shift;
     my $short    = $obj->{'value'};
     my $negative = $short < 0;
@@ -161,118 +153,97 @@ sub ushort { #================================================================
         unshift @bytes, 0 for(1..2-@bytes);
     }
 
-    ### the end
+    ### bytes need a value
     @bytes = (0) unless(@bytes);
-    return @bytes;
-}
 
-sub ushort_list { #===========================================================
+    return wantarray ? @bytes : \@bytes;
 }
 
 sub uchar { #=================================================================
-    return 0xff & shift()->{'value'};
+    my $value = 0xff & shift->{'value'};
+    return wantarray ? ($value) : [$value];
 }
 
 sub vendorspec { #============================================================
 
-    ### init
     my $obj    = shift;
-    my @vendor = map { hex $_ } ($obj->{'value'} =~ /(\w{2})/g);
     my $nested = $obj->{'nested'};
-    my @bytes;
+    my(@vendor, @bytes);
 
     ### check
     return unless(ref $nested eq 'ARRAY');
 
+    @vendor = ether($obj);
+    @bytes  = (8, int(@vendor), @vendor);
+
     TLV:
     for my $tlv (@$nested) {
+        my @value = string($tlv);
         push @bytes, $tlv->{'type'};
-        push @bytes, $tlv->{'length'};
-        push @bytes, $tlv->{'value'};
+        push @bytes, int(@value);
+        push @bytes, @value;
     }
 
-    ### the end
-    return 8, 3, @vendor, @bytes;
+    return wantarray ? @bytes : \@bytes;
 }
 
 sub ip { #====================================================================
-    return split /\./, shift()->{'value'};
+    my @value = split /\./, shift->{'value'};
+    return wantarray ? @value : [@value];
 }
 
 sub ether { #=================================================================
 
-    ### init
-    my $string = shift()->{'value'};
+    my $obj    = shift or return;
+    my $string = $obj->{'value'};
+
+    return unless(defined $string);
 
     ### numeric
     if($string =~ /^\d+$/) {
-        return int_to_bytes({ value => $string });
+        return uint({ value => $string });
     }
 
     ### hex
-    elsif($string =~ /^(?:0x)?[0-9a-f]+$/i) {
-        return int_to_bytes({ value => hex($string) });
+    elsif($string =~ /^(?:0x)?([0-9a-f]+)$/i) {
+        return hexstr({ value => $1 });
     }
 }
 
-sub oid { #===================================================================
-    return snmp_oid(shift()->{'value'});
-}
-
 sub string { #================================================================
 
-    ### init
-    my $string = shift()->{'value'};
+    my $obj    = shift;
+    my $string = $obj->{'value'};
 
     ### hex
-    if($string =~ /^0x[0-9a-f]+$/i) {
-        return int_to_bytes({ value => hex($string) });
+    if($string =~ /^0x([0-9a-f]+)$/i) {
+        return hexstr({ value => $1 });
     }
 
     ### normal
     else {
-        return map { ord $_ } split //, $string;
+        my @ret = map { ord $_ } split //, $string;
+        return wantarray ? @ret : \@ret;
     }
 }
 
-sub strzero { #===============================================================
-    uchar(@_);
-}
-
 sub hexstr { #================================================================
 
-    ### init
-    my $ether = shift()->{'value'};
-
-    ### numeric
-    if($ether =~ /^\d+$/) {
-        return int_to_bytes({ value => $ether });
-    }
-
-    ### hex
-    elsif($ether =~ /^(?:0x)?[0-9a-f]+$/i) {
-        return int_to_bytes({ value => hex($ether) });
-    }
-}
-
-sub int_to_bytes { #==========================================================
-
-    ### init
-    my $string = shift()->{'value'};
+    my $value = shift->{'value'} || '';
     my @bytes;
 
-    while($string) {
-        my $value = $string & 0xff;
-        $string >>= 8;
-        unshift @bytes, $value;
+    if($value =~ /^(?:0x)?([0-9a-f]+)$/i) {
+        while($value) {
+            $value =~ s/(\w{1,2})$//;
+            unshift @bytes, hex $1;
+        }
     }
 
-    ### the end
-    return @bytes;
+    return wantarray ? @bytes: \@bytes;
 }
 
 sub mic { #===================================================================
-    return; # should not get encoded
+    return;
 }
 
 #=============================================================================
@@ -289,9 +260,7 @@ See DOCSIS::ConfigFile
 
 =head1 FUNCTIONS
 
-=head2 byte_size($arg)
-
-Returns the requested byte-size from DOCSIS::ConfigFile.
+Every function can return either a list or an array-ref.
 
 =head2 snmp_type($arg)
 
@@ -300,37 +269,77 @@ Returns an array-ref to an array with two elements:
  1) The numeric value of the SNMP type.
  2) A reference to the function to encode the value.
 
-=head2 snmp_oid
+=head2 snmp_oid($string)
 
-=head2 snmp_object
+Takes a numeric OID and byte-encodes it.
 
-=head2 bigint
+=head2 snmp_object(\%h)
 
-=head2 uint
+Takes a hash-ref (keys: oid, type, value), and returns a byte-encoded snmp-
+object.
 
-=head2 ushort
+ #-type---length---------value-----type---
+   48,    $total_length,         # object
+   6,     int(@oid),     @oid,   # oid
+   $type, int(@value),   @value, # value
 
-=head2 ushort_list
+=head2 bigint(\%h)
 
-=head2 uchar
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value could be any
+number.
 
-=head2 vendorspec
+=head2 uint(\%h)
 
-=head2 ip
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value has to be an
+unsigned int.
 
-=head2 ether
+=head2 ushort(\%h)
 
-=head2 oid
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value has to be an
+unsigned short int.
 
-=head2 string
+=head2 uchar(\%h)
 
-=head2 strzero
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value has to be an
+unsigned char.
 
-=head2 hexstr
+=head2 vendorspec(\%h)
 
-=head2 int_to_bytes
+Takes a hash-ref, and byte-encodes it.
 
-=head2 mic
+Example of the hash-ref:
+
+ {
+   value  => "0x001337", # vendors ID
+   nested => {
+     type   => "24", # vendor specific type
+     value  => "42", # vendor specific value
+   },
+ }
+
+=head2 ip(\%h)
+
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value need to an IPv4
+address.
+
+=head2 ether(\%h)
+
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value need to be a
+six or twelve byte ethernet address.
+
+=head2 string(\%h)
+
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The string could be
+anything in theory, but is often human-readable or a hex-string (leading 0x)
+
+=head2 hexstr(\%h)
+
+Takes a hash-ref, and byte-encodes C<$h-E<gt>{'value'}>. The value can have
+leading '0x'.
+
+=head2 mic(\%h)
+
+Returns C<undef()>.
 
 =head1 AUTHOR
 
